/* unix_subs.c
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* UNIX versions of subroutines called by IO benchmark

         Modified:  4 April 1989 by Barry Wolman
              1. Add append flag to monitor_system

         Modified: 22 April 1990 by Barry Wolman
              1. Use execvp so that iouser and similar parts of
                 benchmark can be "installed".
              2. Change monitor program to call "iousage" instead
                 of "usage" and to do so in a way that "iousage"
                 can be "installed".

         Modified: 18 May 1990 by Barry Wolman
              1. Reorder args to monitor_usage and add delay parameter

         Modified: 20 June 1990 by Barry Wolman
              1. Rename systimes to cptimes, keep user & sys times
                 separate
*/

#include <stdio.h>
#include "ioincl.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>

/* Allocate 128KB of shared memory for use by the various users
   and return a pointer to it
*/

#define SH_MEM_SIZE     128*1024       /* Size of shared memory segment */
#define SH_MEM_KEY      4280           /* Arbitrary key for shared mem */

int shmid;                      /* Shared memory id and ptr */
char *shm_ptr,*shmat();
extern char *sys_errlist[];

char *get_shared_memory(user)
int user;
{
    key_t key = SH_MEM_KEY;

    if (user <= 0)   /* the control user or ioserver must create the sh mem segment */
        if (( shmid = shmget(key, SH_MEM_SIZE, (IPC_CREAT | IPC_EXCL | 00666))) < 0) {
            fprintf(stderr,"Shared memory allocation failed: %s\n",
             sys_errlist[errno]);
            exit(-1);
        }
    if (( shmid = shmget(key, SH_MEM_SIZE, 0)) < 0) {
        fprintf(stderr,"Shared memory allocation failed for user %d: %s\n",
         user,sys_errlist[errno]);
        exit(-1);
    }
    if ((shm_ptr = shmat(shmid, (char *)0, 0)) == (char *)-1) {
        fprintf(stderr,"Shared memory attach failed for user %d: %s\n",
         user,sys_errlist[errno]);
        exit(-1);
    }
    return(shm_ptr);
}

/* Free shared memory segment.  Users 1-N just detach, removal is finally
   done when user 0 calls us */

int free_shared_memory(user)
int user;
{
    if (user > 0) {

         /* This is not controlling user or ioserver */

         if (shmdt (shm_ptr) < 0) {
             fprintf(stderr,"Shared memory detach failed for user %d: %s\n",
              user,sys_errlist[errno]);
             exit(-1);
         }
    } else {

         /* This is controlling user or ioserver */

         if (shmctl(shmid, IPC_RMID, NULL) < 0) {
             fprintf(stderr,"Shared memory delete failed: %s\n",sys_errlist[errno]);
             exit(-1);
         }
    }
}

/*
   Run specified cmd with args as background process
   Last element of argv must be a NULL
*/

run(cmd,argv)
char *cmd, *argv[];
{
    register int pid;

    if (!(pid = fork())) {
        if (execvp(cmd,argv) < 0) {
            fprintf(stderr,"exec failed for cmd %s: %s\n",cmd,sys_errlist[errno]);
            exit(-1);
        }
    }
    /* Parent doesn't wait - we want child to run in background */
    return(pid);
}

/* Generate background process to monitor system for specified
   duration putting output into specified path
*/

monitor_system(delay,duration,path,append)
int delay;       /* Seconds to wait before starting */
int duration;    /* Seconds to monitor */
char *path;      /* Where to put results */
int append;      /* Append flag */
{
    char *monargv[7];
    char temp1[16],temp2[16];

    sprintf(temp1,"%d",delay);
    sprintf(temp2,"%d",duration);
    monargv[0] = "iousage";
    monargv[1] = temp1;
    monargv[2] = temp2;
    monargv[3] = path;
    monargv[4] = append ? "keep" : "rm";
    monargv[5] = NULL;

    run("iousage",monargv);
}

/* Determine user and system CP times */

#include <sys/times.h>
#include <sys/param.h>

cptimes(userp,sysp)
double *userp;    /* where to put user CP time */
double *sysp;     /* where to put system CP time */
{
    struct tms tm;

    time_t tmp;

    long times();

    times(&tm);

    *userp = ((double) tm.tms_utime)/HZ;
    *sysp  = ((double) tm.tms_stime)/HZ;
}



/* Return size of file in bytes */

fsize(path)
char *path;
{
    struct stat sbuf;

    if ((stat(path,&sbuf)) < 0)
        fprintf(stderr,"fsize: could not stat file %s: %s",path,sys_errlist[errno]);

    return(sbuf.st_size);
}
